from rest_framework import serializers
from clusterCore.models import c123_node, c123_host, c123_cluster, c123_inventory, Task, ClusterSizing, IntegratedService
from clusterCore.enums import DBType, ClusterRoles, OperatingSystem, Severity, Status, Inventory, TaskState, Schedule, DataBaseType, Node_Size_Allowed, IntegratedServiceTypes
from clusterCore.api.serializer import c123_CreateHostSerializer, c123_CreateTempClusterSerializer, object_to_json, FORBID_APP_HOST, RECREATION_CONFLICT, OS_CONFLICT, OS_UNSUPPORTED, PRIMARY_GONE
from clusterCore.exceptions import *
from clusterCore.tasks import drop_backup, license_after_create, create_ssh_connection, remote_command, check_os_version, format_rsa_key, ensure_primary_created, get_fully_qualified_domain_name, format_password
from apscheduler.triggers.interval import IntervalTrigger
from django.utils import timezone
import json
class c123_CreatePGClusterSerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'database_user', 'database_pwd', 'dir_123Cluster', 'ex_params']
    @license_after_create  
    def create(self, validated_data):
        # app_hostname = self.context["request"].META['REMOTE_ADDR']
        # app_hostname = self.context["request"].get_host().split(':')[0
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()
        create_run_linux = True
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        aws_host =  host_data['ex_params'].get('aws_username')
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host or aws_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
       
        hostname = host_data['name']
        version = validated_data["version"]
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
            raise APIException406(FORBID_APP_HOST.format(hostname))
        host_exist = c123_host.objects.filter(name = hostname).exists()
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        pgcluster_exists = c123_node.objects.filter(name = node_name)
        if pgcluster_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.POSTGRES.value, node_name))
        pgcluster_to_update = pgcluster_exists.filter(ind_deleted = True).first()
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = validated_data['ex_params'].get('distributor')
            release = validated_data['ex_params'].get('release')
        
        if (int(version) > 15 and not ((OperatingSystem.CENTOS.value in distributor and int(release) > 7) or (OperatingSystem.UBUNTU.value in distributor and int(release) > 18))):
            raise APIException406(OS_UNSUPPORTED.format(DataBaseType.POSTGRES.name, version, distributor, release))
        
        validated_data['ex_params']['distributor'] = distributor
        validated_data['ex_params']['release'] = release
        client = create_ssh_connection(app_host.name,  app_host.username, app_host.password, app_host.private_key, app_host.ssh_port)
        if isinstance(client, tuple):
            if hasattr(client[1], 'strerror'):
                raise APIException401(client[1].strerror)
            raise APIException401(client[1])       
        
        jsn = json.loads(remote_command(client, "sudo echo $SSH_CLIENT | awk '{print $1}'", True), strict=False)
        if jsn['stderr']:
            app_hostname = self.context["request"].get_host().split(':')[0]
        else:
            app_hostname = jsn['stdout']
        cluster_data = validated_data.pop('cluster')
        cluster_data['created_by'] = current_user
        cluster_data['last_update_by'] = current_user
        cluster_data['type'] = DBType.POSTGRES.value
        
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.POSTGRES.value
        validated_data['cluster_role'] = ClusterRoles.PRIMARY.value
        validated_data['sizing'] = self.context['request'].data.get('sizing')
        pg_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.POSTGRES.value)  
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = pg_sys.small_memory
                cpu_cores = pg_sys.small_cpu
                disk_gb = pg_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = pg_sys.medium_memory
                cpu_cores = pg_sys.medium_cpu
                disk_gb = pg_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = pg_sys.large_memory
                cpu_cores = pg_sys.large_cpu
                disk_gb = pg_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = pg_sys.very_large_memory
                cpu_cores = pg_sys.very_large_cpu
                disk_gb = pg_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data['database_pwd'] = format_password(validated_data['database_pwd'])
        validated_data["ind_deleted"] = False  
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        own_cluster = c123_cluster.objects.filter(name = cluster_data["name"]).first()
        #if recreated primary has not registered cluster for self in the past
        if not own_cluster:
            own_cluster, created = c123_cluster.objects.get_or_create(**cluster_data)
        if not own_cluster.ex_params:
            own_cluster.ex_params = {}
        own_cluster.ex_params["new_basebackup"] = True
        own_cluster.ex_params['primary_count'] = 1
        own_cluster.save()
            
        validated_data["cluster"] = own_cluster
        if pgcluster_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=pgcluster_to_update.pk).update(**validated_data)
            pgcluster = c123_node.objects.get(pk=pgcluster_to_update.pk)
        else:    
            pgcluster = c123_node.objects.create(**validated_data)
        
        pgcluster.ex_params['system_identifier'] = ''
        pgcluster.ex_params['ind_haproxy'] = False
        pgcluster.ex_params["pg_primary_id"] = pgcluster.pk
        pgcluster.ex_params["pg_password"] = format_password(validated_data["ex_params"]["pg_password"])
        pgcluster.save()
        
        if self.context["request"].data.get("auto_delete"):
            pgcluster.auto_delete =  pgcluster.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            pgcluster.save()

        repository =  pgcluster.ex_params["repository"]
    
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = pgcluster))
        else:
            c123_inventory.objects.filter(node = pgcluster).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = pgcluster.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = pgcluster.type, ex_params__node_version = pgcluster.version).delete()
        

        from django.conf import settings
        app_host_pg_password = settings.DATABASES['default']['PASSWORD']
        log_archive_path = f'/123cluster/archive_logs/{pgcluster.type}/{pgcluster.cluster.name}'
        tmp_data = {'node_data': object_to_json(pgcluster), 'host_data': object_to_json(pgcluster.host), 'app_host': object_to_json(app_host), 'app_hostname': app_hostname, 'create_run_linux': create_run_linux, 'repository': repository, 'created_by': current_user.id, 'app_host_pg_password': app_host_pg_password, 'log_archive_path': log_archive_path, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'disk_gb': disk_gb}
        if proxmox_host or aws_host:
            pgcluster.ex_params['service_id'] = service.pk
            pgcluster.save()
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='install_postgres_standalone_task',
                                    displayment='creating pg node {}'.format(pgcluster.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=pgcluster.id,
                                    org_id=pgcluster.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        
        from django.conf import settings
        from clusterCore.scheduler import UpdatePgClusterRoles, RemovePgSlots
        settings.scheduler.add_job(UpdatePgClusterRoles,IntervalTrigger(seconds=80, timezone=settings.TIME_ZONE), replace_existing=True, next_run_time=timezone.now() + timezone.timedelta(minutes = 5), id= f'update_pg_roles_{own_cluster.id}',kwargs={"cluster_id": pgcluster.cluster.id})
        settings.scheduler.add_job(RemovePgSlots,IntervalTrigger(seconds=80, timezone=settings.TIME_ZONE), next_run_time=timezone.now() + timezone.timedelta(minutes = 5),  id=f'remove_pg_slots_{own_cluster.id}', kwargs={"cluster_id": own_cluster.id}, replace_existing=True)
        
        return pgcluster, task

class c123_CreateHAProxySerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'ex_params']
    @license_after_create
    def create(self, validated_data):
        
        create_run_linux = True
        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")
      
        hostname = host_data['name']
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
             raise APIException406(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()
        host_exists = c123_host.objects.filter(name = hostname).first()
        cluster = c123_cluster.objects.get(pk = self.context["request"].data['cluster']['id'])
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        haproxy_exists = c123_node.objects.filter(name = node_name)
        if haproxy_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.HAPROXY.value, node_name))
        haproxy_to_update = haproxy_exists.filter(ind_deleted = True).first()
        
        current_user = self.context['request'].user
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = validated_data['ex_params'].get('distributor')
            release = validated_data['ex_params'].get('release')
        
      
        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.HAPROXY.value
        validated_data['cluster_role'] = ClusterRoles.BALANCER.value
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        validated_data["cluster"] = cluster
        validated_data["ind_deleted"] = False
        private_key = format_rsa_key(host_data['private_key'].strip())
        if host_exists:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
          
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        
        if haproxy_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=haproxy_to_update.pk).update(**validated_data)
            haproxy = c123_node.objects.get(pk=haproxy_to_update.pk) 
        else:    
            haproxy = c123_node.objects.create(**validated_data)
        
        if self.context["request"].data.get("auto_delete"):
            haproxy.auto_delete =  haproxy.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            haproxy.save()
        
        repository = self.context["request"].data.get("repository")
        cluster_nodes = c123_node.objects.filter(ind_deleted = False, cluster = haproxy.cluster).exclude(type = DBType.HAPROXY.value)
        hosts_and_nodes = [{"host": object_to_json(i.host), "node": object_to_json(i)} for i in cluster_nodes]
        has_other_haproxies = c123_node.objects.filter(host = haproxy.host, type = DBType.HAPROXY.value, ind_deleted = False).exclude(pk = haproxy.pk).exists()
        haproxy_exists = self.context['request'].data.get('haproxy_exists')
        tmp_data = {'host_haproxy_data': object_to_json(haproxy.host), 'app_host': object_to_json(app_host), 'node_haproxy_data': object_to_json(haproxy), 'hosts_and_nodes': hosts_and_nodes, 'haproxy_exists': haproxy_exists, 'has_other_haproxies': has_other_haproxies,  'create_run_linux': create_run_linux, 'repository': repository, 'created_by': current_user.id}
        if proxmox_host:
            tmp_data['service_data'] = object_to_json(service)
        task = Task.objects.create(task='install_haproxy_task',
                                    displayment='creating haproxy {}'.format(haproxy.name),
                                    target_type=DBType.POSTGRES.value,
                                    target_id=haproxy.id,
                                    org_id=haproxy.org_id,
                                    status=TaskState.PENDING.value,
                                    schedule=Schedule.NOW.value,
                                    parameters=json.dumps(tmp_data),
                                    created_by=current_user)
        return haproxy, task
class c123_AddPGStandbySerializer(serializers.HyperlinkedModelSerializer):
    
    host = c123_CreateHostSerializer()
    cluster = c123_CreateTempClusterSerializer()
    class Meta:
        model = c123_node
        fields = ['name', 'cluster', 'host', 'port', 'version', 'dir_123Cluster', 'ex_params']
    
    @license_after_create 
    def create(self, validated_data):
        # app_hostname = self.context["request"].META['REMOTE_ADDR']
        # app_hostname = self.context["request"].get_host().split(':')[0]
        
        create_run_linux = True


        host_data = validated_data.pop('host')
        if not host_data.get('ex_params'):
            host_data['ex_params'] = {}
        
        proxmox_host = validated_data['ex_params'].get('proxmox')
        if proxmox_host:
            service_name = validated_data['ex_params'].get('service_name')
            service = IntegratedService.objects.filter(name = service_name, type = IntegratedServiceTypes.PROXMOX.name).first()
            if not service:
                raise APIException404(f"{service_name} is not found")

        hostname = host_data['name'] 
        on_app_host = c123_host.objects.filter(name = hostname, ex_params__myself = True).exists()
        if on_app_host:
             raise APIException406(FORBID_APP_HOST.format(hostname))
        app_host = c123_host.objects.filter(ex_params__myself = True, ind_deleted = False).first()
        host_data['name'] = hostname
        node_name = f"{hostname}:{validated_data['port']}"
        validated_data['name'] = node_name
        
        primary_id = validated_data['ex_params']['pg_primary_id']
        primary_cluster = c123_node.objects.filter(type = DBType.POSTGRES.value, pk=primary_id, ind_deleted = False).first()
        if not primary_cluster:
            raise APIException410(PRIMARY_GONE.format(primary_id))
        ensure_primary_created(primary_cluster)
        cluster_ids = c123_node.objects.filter(cluster = primary_cluster.cluster).values_list('id', flat=True)
        repmgr_creating = Task.objects.filter(target_id__in = cluster_ids, task = 'create_repmgr_task', status__in=[TaskState.PENDING.value, TaskState.RUNNING.value]).exists()
        if repmgr_creating:
            raise APIException403(f'Cluster of primary {primary_cluster.name} has repmgr creating.Unable to add new standbys')
  
        current_user = self.context['request'].user 
        host_data['created_by'] = current_user
        host_data['last_update_by'] = current_user
        host_data['password'] = format_password(host_data['password'])
           
        host_primary = primary_cluster.host
        (distributor_primary, release_primary) = check_os_version(host_primary.__dict__)
        if OperatingSystem.UNKNOWN.value in distributor_primary:
            raise APIException401(release_primary['stderr'])
        
        if not proxmox_host:
            (distributor, release) = check_os_version(host_data)
            if OperatingSystem.INVALID.value in distributor:
                raise APIException401(release['stderr'])
        else:
            distributor = distributor_primary
            release = release_primary
        
        if distributor_primary != distributor or release_primary != release:
            raise APIException406(OS_CONFLICT.format(distributor_primary, release_primary,distributor,release))
        
        client = create_ssh_connection(app_host.name, app_host.username, app_host.password, app_host.private_key, app_host.ssh_port)
        if isinstance(client, tuple):
            if hasattr(client[1], 'strerror'):
                raise APIException401(client[1].strerror)
            raise APIException401(client[1])
        

        jsn = json.loads(remote_command(client, "sudo echo $SSH_CLIENT | awk '{print $1}'", True), strict=False)
        if jsn['stderr']:
            app_hostname = self.context["request"].get_host().split(':')[0]
        else:
            app_hostname = jsn['stdout']

        validated_data['created_by'] = current_user
        validated_data['last_update_by'] = current_user
        validated_data['status'] = Status.CREATING.value
        validated_data['severity'] = Severity.NOT_OK.value 
        validated_data['type'] = DBType.POSTGRES.value
        validated_data['cluster_role'] = ClusterRoles.STANDBY.value
        validated_data['database_user'] = primary_cluster.database_user
        validated_data['database_pwd'] = primary_cluster.database_pwd
        validated_data['sizing'] =  self.context['request'].data.get('sizing')
        pg_sys, created = ClusterSizing.objects.get_or_create(db_type = DBType.POSTGRES.value)
        match validated_data['sizing']:
            case Node_Size_Allowed.SMALL.name:
                ram_gb = pg_sys.small_memory
                cpu_cores = pg_sys.small_cpu
                disk_gb = pg_sys.small_disk
            case Node_Size_Allowed.MEDIUM.name:
                ram_gb = pg_sys.medium_memory
                cpu_cores = pg_sys.medium_cpu
                disk_gb = pg_sys.medium_disk
            case Node_Size_Allowed.LARGE.name:
                ram_gb = pg_sys.large_memory
                cpu_cores = pg_sys.large_cpu
                disk_gb = pg_sys.large_disk
            case Node_Size_Allowed.VERY_LARGE.name:
                ram_gb = pg_sys.very_large_memory
                cpu_cores = pg_sys.very_large_cpu
                disk_gb = pg_sys.very_large_disk
            case _:
                ram_gb = 1
                cpu_cores = 1
                disk_gb = 10 
        validated_data['cluster'] = primary_cluster.cluster
        validated_data['ind_deleted'] = False
        private_key = format_rsa_key(host_data['private_key'].strip())
        host_exist = c123_host.objects.filter(name = hostname).exists()
        if host_exist:
            c123_host.objects.filter(name=hostname).update(**host_data)
            host = c123_host.objects.get(name=hostname)
            create_run_linux = False
        else:
            host = c123_host.objects.create(**host_data)
        host.private_key = private_key
        host.save()
        validated_data['host'] = host
        pgstandby_exists = c123_node.objects.filter(name = node_name)
        if pgstandby_exists.filter(ind_deleted = False, host__name = hostname).exists():
                raise APIException409(RECREATION_CONFLICT.format(DataBaseType.POSTGRES.value, node_name))
        pgstandby_to_update = pgstandby_exists.filter(ind_deleted = True).first()
        if pgstandby_to_update:
            validated_data["created_date"] = timezone.now()
            c123_node.objects.filter(pk=pgstandby_to_update.pk).update(**validated_data)
            pgstandby = c123_node.objects.get(pk=pgstandby_to_update.pk)     
        else:    
            pgstandby = c123_node.objects.create(**validated_data)
        if self.context["request"].data.get("auto_delete"):
            pgstandby.auto_delete =  pgstandby.created_date + timezone.timedelta(days=float(self.context["request"].data.get("auto_delete_days")))
            pgstandby.save()
        
        haproxy_exists = c123_node.objects.filter(cluster = primary_cluster.cluster, type = DBType.HAPROXY.value, ind_deleted = False).exists()
        pgstandby.ex_params['system_identifier'] = ''
        pgstandby.ex_params['ind_haproxy'] = haproxy_exists
        pgstandby.ex_params['distributor'] = distributor
        pgstandby.ex_params['release'] = release
        pgstandby.save()
        repmgr_pwd =  format_password(validated_data["ex_params"]["repmgr_pwd"])
        cluster_standbys = c123_node.objects.filter(type = DBType.POSTGRES.value, cluster = primary_cluster.cluster, cluster_role = ClusterRoles.STANDBY.value, ind_deleted = False).exclude(pk = pgstandby.pk)
        standbys = [{"host": object_to_json(i.host), "node_id": i.pk, "node_port": i.port, "db_usr": i.database_user, "db_pwd": i.database_pwd, "data_directory": i.dir_123Cluster} for i in cluster_standbys]
        other_standbys_exists = cluster_standbys.exclude(pk = pgstandby.pk).exists()
        task_name = "add_standby_repmgr_task" if other_standbys_exists else "create_repmgr_task"
        
        primary_cluster.ex_params["repmgr_pwd"] = repmgr_pwd
        primary_cluster.save()
            
        pgstandby.ex_params["repmgr_pwd"] = primary_cluster.ex_params["repmgr_pwd"]
        pgstandby.cluster = primary_cluster.cluster
        pgstandby.ex_params["pg_password"] = primary_cluster.ex_params["pg_password"]
        pgstandby.ex_params["repository"] = primary_cluster.ex_params["repository"]
        pgstandby.ex_params["ssl"] = primary_cluster.ex_params["ssl"]
        pgstandby.ex_params["certificate_authentication"] = primary_cluster.ex_params["certificate_authentication"]
        pgstandby.ex_params['system_identifier'] = primary_cluster.ex_params['system_identifier']
        pg_password = primary_cluster.ex_params["pg_password"]
        repository = primary_cluster.ex_params["repository"]
        pgstandby.save()
        if self.context["request"].data.get("drop_inventories"):
            drop_backup(c123_inventory.objects.filter(node = pgstandby))
        else:
            c123_inventory.objects.filter(node = pgstandby).exclude(type = Inventory.EXPORT.value, ex_params__belong_to = pgstandby.type).exclude(type = Inventory.BACKUP.value, ex_params__belong_to = pgstandby.type, ex_params__node_version = pgstandby.version).delete()
        tmp_data = {'primary_data': object_to_json(primary_cluster), 'standby_data': object_to_json(pgstandby), 'standbys': standbys, 'host_primary': object_to_json(host_primary), 'host_standby': object_to_json(pgstandby.host), 'app_host': object_to_json(app_host), 'app_hostname': app_hostname, 'create_run_linux': create_run_linux, 'repository': repository, "pg_password": pg_password, 'ram_gb': ram_gb, 'cpu_cores': cpu_cores, 'disk_gb': disk_gb, 'created_by': current_user.id}
        if proxmox_host:
            pgstandby.ex_params['service_id'] = service.pk
            pgstandby.save()
            tmp_data['service_data'] = object_to_json(service)
        haproxies = c123_node.objects.filter(cluster = pgstandby.cluster, type = DBType.HAPROXY.value, ind_deleted = False)
        haproxy_exists = haproxies.exists()
        tmp_data['haproxy_exists'] = haproxy_exists
        if haproxy_exists:
            tmp_data['haproxy_data'] = [{'port': i.port, 'host': object_to_json(i.host), 'has_other_on_host': c123_node.objects.filter(host = i.host, type = DBType.HAPROXY.value, ind_deleted = False).exclude(pk = i.pk).exists()} for i in haproxies]
        
        from django.conf import settings
        app_host_pg_password = settings.DATABASES['default']['PASSWORD']
        tmp_data['app_host_pg_password'] = app_host_pg_password
       

        task = Task.objects.create(task=task_name,
                            displayment='adding pg standby {}'.format(pgstandby.name),
                            target_type=DBType.POSTGRES.value,
                            target_id=pgstandby.id,
                            org_id=pgstandby.org_id,
                            status=TaskState.PENDING.value,
                            schedule=Schedule.NOW.value,
                            parameters=json.dumps(tmp_data),
                            created_by=current_user)
        return pgstandby, task